@@ -3,7 +3,7 @@
 //
 //  This file is part of Linterna Mágica
 //
-//  Copyright (C) 2010, 2011  Ivaylo Valkov <ivaylo@e-valkov.org>
+//  Copyright (C) 2010, 2011, 2012  Ivaylo Valkov <ivaylo@e-valkov.org>
 //  Copyright (C) 2010  Anton Katsarov <anton@katsarov.org>
 //
 //  The JavaScript code in this page (or file) is free software: you
@@ -106,6 +106,7 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
 		break;
 	    case '13':
 	    case '17':
+	    case '36':
 		label += " 3GP";
 		break;
 	    default:
@@ -156,6 +157,7 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
 		break;
 	    case '13':
 	    case '17':
+	    case '36':
 		more_info += "MPEG-4 Visual, AAC";
 		break;
 	    default:
@@ -196,7 +198,7 @@ LinternaMagica.prototype.detect_youtube_flash_upgrade = function(object_data)
 
     // Fancy flash upgrade message. The element with id flash-upgrade
     // might be obsolete now.
-    var watch_player = document.getElementById("watch-player");
+    var watch_player = document.getElementById("watch7-player");
     var fancy_alert = null;
 
     if(watch_player && watch_player.hasAttribute("class") &&
@@ -212,12 +214,12 @@ LinternaMagica.prototype.detect_youtube_flash_upgrade = function(object_data)
     }
     
 
-    // With default timeout 500mS this will be 6 sec. Stop checking and insert.    
+    // With default timeout 3000mS this will be >10 sec. Stop checking and insert.    
     // Might be flashblock
-    if (document.getElementById("flash-upgrade") ||
+    if (document.getElementById("movie_player") ||
 	document.getElementById("movie_player-html5") ||
 	fancy_alert ||
-	this.youtube_flash_upgrade_counter >= 12 )
+	this.youtube_flash_upgrade_counter >= 5 )
     {
 	clearInterval(this.youtube_flash_upgrade_timeout);
 
@@ -229,7 +231,7 @@ LinternaMagica.prototype.detect_youtube_flash_upgrade = function(object_data)
 	this.log("LinternaMagica.detect_youtube_flash_upgrade:\n"+
 		 "Creating video object.",2);
 
-	this.create_video_object(object_data);
+	setTimeout(this.create_video_object(object_data), 1000);
     }
 }
 
@@ -265,7 +267,7 @@ LinternaMagica.prototype.extract_youtube_fmt_url_map = function()
 	{
 	    // Usually the links have the following pattern
 	    // (itag=fmt_id)*url=URL&type=video/...&(itag=fmt_id)*
-	    var link = fmt[url].match(/url=([^&]+)/);
+	    var link = fmt[url].match(/(url|conn)=([^&]+)/);
 	    var fmt_id = fmt[url].match(/itag=([0-9]+)/);
 	    var sig = fmt[url].replace(/\\u0026/g, '&').match(/sig=[^&]+/);
 		sig = sig[sig.length-1].replace(/sig/,'signature');
@@ -274,6 +276,10 @@ LinternaMagica.prototype.extract_youtube_fmt_url_map = function()
 	    {
 		links++;
 		link = unescape(link[link.length-1]);
+		// Live streams support. 
+		// See bugs #36759:
+		// https://savannah.nongnu.org/bugs/?36759
+		link = link.replace(/\\u0026stream=/, '/');
 		link = link.split(/\\u0026/)[0];
 
 		map[fmt_id[fmt_id.length-1]] =  link+"&"+sig;
@@ -334,13 +340,20 @@ LinternaMagica.prototype.sites["youtube.com"].skip_link_extraction = function()
     return false;
 }
 
+LinternaMagica.prototype.sites["youtube.com"].skip_video_id_extraction = function()
+{
+    this.log("LinternaMagica.sites.skip_video_id_extraction:\n"+
+	     "Skipping video_id extraction in YouTube.",4);
+    return false;
+}
+
+
 // Extracts data for the flash object in youtube from a script
 LinternaMagica.prototype.sites["youtube.com"].extract_object_from_script =
 function()
 {
     var data = this.script_data;
-    if (!data.match(/var\s*swfConfig/) &&
-	!data.match(/yt\.setConfig/))
+    if (!data.match(/yt\.playerConfig =/))
     {
  	return null;
     }
@@ -379,9 +392,12 @@ function()
 	embed_id= embed_id[embed_id.length-1];
     }
 
+    var p = this.get_first_element_by_class("flash-player");
+    p = p ? p : this.get_first_element_by_class("html5-player");
+
+
     if (!width || !height)
     {
-	var p = document.getElementById("watch-player");
 	height = p ? p.clientHeight : null;
 	width  = p ? p.clientWidth : null;
     }
@@ -433,7 +449,7 @@ function()
 	linterna_magica_id =
 	    this.mark_flash_object("extracted-from-script");
 
-	object_data.parent = document.getElementById("watch-player");
+	object_data.parent =  p;
     }
 
     object_data.linterna_magica_id = linterna_magica_id;
@@ -469,7 +485,7 @@ function(object_data)
 	this.youtube_flash_upgrade_timeout = setInterval(
 	    function() {
 		self.detect_youtube_flash_upgrade.apply(self,[data]);
-	    }, 500);
+	    }, 3000);
     }
     
     return false;
@@ -478,6 +494,27 @@ function(object_data)
 LinternaMagica.prototype.sites["youtube.com"].css_fixes =
 function(object_data)
 {
+
+    // Sometimes when flash is installed the flash video object does
+    // not have (at all or the right one) linterna_magica_id. Usually
+    // the other objects if any are useless. This renders both LM and
+    // the flash interface. The code bellow tries to avoid it. Reason
+    // *unknown*.
+
+    this.log("LinternaMagica.youtube.css_fixes:\n "+
+	     "Harvesting (possible) lost flash video object with "+
+	     "linterna_magica_id "+ object_data.linterna_magica_id);
+
+    var movie_player = document.getElementById('movie_player');
+    if (movie_player) {
+	movie_player.linterna_magica_id = object_data.linterna_magica_id;
+    }
+
+    if (this.priority.self > this.priority.plugin)
+    {
+	this.hide_flash_video_object(object_data.linterna_magica_id);
+    }
+
     if (document.getElementById("playnav-playview"))
     {
 	// In channels/user pages in YouTube the web controlls are
@@ -549,30 +586,6 @@ function(object_data)
 	
     }
 
-    // Sometimes the toggle plugin button that should be bellow the
-    // HTML5/Flash player shows on top. The following tries to fix
-    // this.  Most of the time the bug occurs in Epiphany.s
-
-    var id = object_data.linterna_magica_id;
-    var toggle_plugin = document.getElementById("linterna-magica-toggle-plugin-"+id);
-    
-    if (toggle_plugin)
-    {
-	var lm = document.getElementById("linterna-magica-"+id);
-	var h_lm = parseInt(lm.style.getPropertyValue("height"));
-
-	toggle_plugin = toggle_plugin.parentNode;
-	var display = toggle_plugin.style.getPropertyValue("display");
-	var top = toggle_plugin.offsetTop;
-
-	if (!display && !top)
-	{
-	    toggle_plugin.style.setProperty("top", 
-					    (h_lm +30)+"px",
-					    "important");
-	}
-    }
-
     // When YouTube's HTML5 player detects that the browser does not
     // have support for any clip, a warning div is displayed. It must
     // be hidden or it displaces Linterna Magica.
@@ -604,6 +617,36 @@ function(object_data)
 	    // will not be displaced.
 	    html5_wrapper.style.setProperty('display', "none", "important");
 
+	    // If the HTML5 player has a fallback CSS class and LM can
+	    // access it it means there is no other option for
+	    // playback. We take over the player.
+	    //
+	    // See bug 37630
+	    // https://savannah.nongnu.org/bugs/index.php?37630
+	    if (this.priority.html5 > this.priority.self)
+ 	    {
+		// It should be available, because it is used above in
+		// this function
+		var id = object_data.linterna_magica_id;
+
+		this.show_lm_interface(id);
+	    
+		// It should be available, because it is used above in
+		// this functon.
+		toggle_plugin = 
+		    document.getElementById("linterna-magica-"+
+					    "toggle-plugin-"+id);
+
+		var pn = toggle_plugin.parentNode.parentNode;
+		pn.removeChild(toggle_plugin.parentNode);
+
+		toggle_plugin_header =
+		    document.getElementById("linterna-magica-"+
+					    "toggle-plugin-header-"+id);
+		
+		toggle_plugin_header.parentNode.removeChild(toggle_plugin_header);
+	    }
+	    
 	    // Hide site controls and video container. Should not
 	    // overlap with Linterna Magica now. Fixes the overlapping
 	    // of HTML5 player and LM.
@@ -623,5 +666,24 @@ function(object_data)
 	}
     }
 
+    // Without this the toggle plugin stays hidden below the title of
+    // the clip
+    var watch7 = document.getElementById('watch7-player');
+    if (watch7)
+    {
+	watch7.style.setProperty("height", 
+				 (parseInt(object_data.outer_height)+
+				  24)+"px", "important");
+
+	var movie_player = document.getElementById("movie_player");
+	if (movie_player)
+	{
+	    movie_player.style.setProperty("height", 
+					   parseInt(object_data.outer_height)+
+					   "px", "important");
+	}
+    }
+
     return false;
 }
+
